// ------------------------------------------------------------------------------
// BallCollision - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Ball.java
// Teh balls that collide and bounce
// ------------------------------------------------------------------------------

class Ball
{
  float r;      // radius
  float m;      // mass
  
  float x;      // position
  float y;
  
  float vx;     // velocity
  float vy;
  
  Ball()
  {
    r = random(15,70);
    m = r;
    x = random(r,width-r);
    y = 0;
    vx = random(-5,5);
  }
  
  void fall()
  {
    // Vertical force of gravity
    vy += g;
  }
  
  void bounce()
  {
    // Bounce off the walls
    if ( y + vy + r > height-3 )
    {
      y = height - r - 3;
      vx *= f;
      vy *= -b;
    }
    if ( y + vy - r < 3 )
    {
      y = r + 3;
      vx *= f;
      vy *= -b;
    }
    if ( x + vx + r > width-3 )
    {
      x = width - r - 3;
      vx *= -b;
      vy *= f;
    }
    if ( x + vx - r < 3 )
    {
      x = r + 3;
      vx *= -b;
      vy *= f;
    }
  }
  
  void collide()
  {
    // Bounce off each other
    for ( int i=0; i<num; i++ )
    {
      float X = balls[i].x;
      float Y = balls[i].y;
      float R = balls[i].r;
      float M = balls[i].m;
      
      float deltax = X-x;
      float deltay = Y-y;
      float d = sqrt(pow(deltax,2)+pow(deltay,2));
      
      if ( d < r + R + 2 && d > 0 )
      {
        // We add the 2 to compensate for the
        // thickness of the balls' stroke
        float dD = r + R - d + 2;
        float theta = atan2(deltay,deltax);
        
        vx += -dD*cos(theta)*M/(m+M);
        vy += -dD*sin(theta)*M/(m+M);
        
        vx *= b;
        vy *= b;
      }
    }
  }
  
  void move()
  {
    x += vx;
    y += vy;
  }
  
  void display()
  {
    fill(240,240,240);
    ellipse(x,y,2*r,2*r);
  }
}
