import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class BallCollision extends PApplet {// ------------------------------------------------------------------------------
// BallCollision - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// BalCollision.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

int t = -1;
int num = 15;
Ball[] balls = new Ball[num];

float g = 0.20f;                            // Gravity
float b = 0.95f;                            // Bounciness
float f = 0.50f;                            // Friction

public void setup()
{
  size(800,420);
  framerate(30);
  smooth();
  
  stroke(0);
  strokeWeight(2);
  
  reset();
}

public void draw()
{
  background(255,255,255);
  
  if ( mousePressed == true )
  {
    drag();
  }
  
  if ( keyPressed )
  {
    if ( key == '=' || key == '-' || key == '+' )
    {
      float k = 1;
    
      if ( key == '=' || key == '+' )
      {  
        k = 1.01f;
      }
      else
      {
        k = 0.99f;
      }
    
      for ( int i=0; i<num; i++ )
      {
        balls[i].r *= k;
      }
    }
    if ( key == ' ' )
    {
      shake();
    }
  }

  for ( int i=0; i<num; i++ )
  {
    balls[i].fall();
    balls[i].bounce();
    balls[i].collide();
    balls[i].move();
    balls[i].display();
  }
}

public void mousePressed()
{
  for ( int i=0; i<num; i++ )
  {
    if ( abs(mouseX-balls[i].x) < balls[i].r && abs(mouseY-balls[i].y) < balls[i].r )
    {
      t = i;
    }
  }
}

public void drag()
{
  if ( t >= 0 )
  {
    balls[t].x = mouseX;
    balls[t].y = mouseY;
  
    balls[t].vy = 0;
    balls[t].vx = 0;
  }
}

public void reset()
{ 
  for ( int i=0; i<num; i++ )
  {
    balls[i] = new Ball();
  }
}

public void shake()
{
  for ( int i=0; i<num; i++ )
  {
    balls[i].vx += random(-5,5);
    balls[i].vy += random(-5,5);
  }
}

// ------------------------------------------------------------------------------
// BallCollision - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Ball.java
// Teh balls that collide and bounce
// ------------------------------------------------------------------------------

class Ball
{
  float r;      // radius
  float m;      // mass
  
  float x;      // position
  float y;
  
  float vx;     // velocity
  float vy;
  
  Ball()
  {
    r = random(15,70);
    m = r;
    x = random(r,width-r);
    y = 0;
    vx = random(-5,5);
  }
  
  public void fall()
  {
    // Vertical force of gravity
    vy += g;
  }
  
  public void bounce()
  {
    // Bounce off the walls
    if ( y + vy + r > height-3 )
    {
      y = height - r - 3;
      vx *= f;
      vy *= -b;
    }
    if ( y + vy - r < 3 )
    {
      y = r + 3;
      vx *= f;
      vy *= -b;
    }
    if ( x + vx + r > width-3 )
    {
      x = width - r - 3;
      vx *= -b;
      vy *= f;
    }
    if ( x + vx - r < 3 )
    {
      x = r + 3;
      vx *= -b;
      vy *= f;
    }
  }
  
  public void collide()
  {
    // Bounce off each other
    for ( int i=0; i<num; i++ )
    {
      float X = balls[i].x;
      float Y = balls[i].y;
      float R = balls[i].r;
      float M = balls[i].m;
      
      float deltax = X-x;
      float deltay = Y-y;
      float d = sqrt(pow(deltax,2)+pow(deltay,2));
      
      if ( d < r + R + 2 && d > 0 )
      {
        // We add the 2 to compensate for the
        // thickness of the balls' stroke
        float dD = r + R - d + 2;
        float theta = atan2(deltay,deltax);
        
        vx += -dD*cos(theta)*M/(m+M);
        vy += -dD*sin(theta)*M/(m+M);
        
        vx *= b;
        vy *= b;
      }
    }
  }
  
  public void move()
  {
    x += vx;
    y += vy;
  }
  
  public void display()
  {
    fill(240,240,240);
    ellipse(x,y,2*r,2*r);
  }
}
static public void main(String args[]) {   PApplet.main(new String[] { "BallCollision" });}}